@@ -135,29 +135,22 @@ static int get_shift(int timeres, const char *buf)
 {
     int sign = 1;
     int a = 0, b = 0, c = 0, d = 0;
-    int64_t ret;
 #define SSEP "%*1[.:]"
     int n = sscanf(buf, "%d"SSEP"%d"SSEP"%d"SSEP"%d", &a, &b, &c, &d);
 #undef SSEP
 
-    if (a == INT_MIN)
-        return 0;
-
     if (*buf == '-' || a < 0) {
         sign = -1;
         a = FFABS(a);
     }
 
-    ret = 0;
     switch (n) {
-    case 4: ret = sign * (((int64_t)a*3600 + b*60 + c) * timeres + d);
-    case 3: ret = sign * ((         (int64_t)a*60 + b) * timeres + c);
-    case 2: ret = sign * ((                (int64_t)a) * timeres + b);
+    case 4: return sign * ((a*3600 + b*60 + c) * timeres + d);
+    case 3: return sign * ((         a*60 + b) * timeres + c);
+    case 2: return sign * ((                a) * timeres + b);
     }
-    if ((int)ret != ret)
-        ret = 0;
 
-    return ret;
+    return 0;
 }
 
 static int jacosub_read_header(AVFormatContext *s)
@@ -228,17 +221,14 @@ static int jacosub_read_header(AVFormatContext *s)
             }
             av_bprintf(&header, "#S %s", p);
             break;
-        case 'T': { // ...but must be placed after TIMERES
-            int64_t timeres = strtol(p, NULL, 10);
-            if (timeres <= 0 || timeres > UINT32_MAX) {
+        case 'T': // ...but must be placed after TIMERES
+            jacosub->timeres = strtol(p, NULL, 10);
+            if (!jacosub->timeres)
                 jacosub->timeres = 30;
-            } else {
-                jacosub->timeres = timeres;
+            else
                 av_bprintf(&header, "#T %s", p);
-            }
             break;
         }
-        }
     }
 
     /* general/essential directives in the extradata */
